package java.util.concurrent;

import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Collection;

public abstract class AbstractExecutorService implements ExecutorService
{
    public AbstractExecutorService() {
    }
    
    protected <T> RunnableFuture<T> newTaskFor(Runnable runnable, T t) {
        return new FutureTask(runnable, t);
    }
    
    protected <T> RunnableFuture<T> newTaskFor(Callable<T> callable) {
        return new FutureTask(callable);
    }
    
    @Override
    public Future<?> submit(Runnable runnable) {
        if (runnable == null) {
            throw new NullPointerException();
        }
        RunnableFuture<Object> task = this.newTaskFor(runnable, null);
        this.execute(task);
        return task;
    }
    
    @Override
    public <T> Future<T> submit(Runnable runnable, T t) {
        if (runnable == null) {
            throw new NullPointerException();
        }
        RunnableFuture<T> task = this.newTaskFor(runnable, t);
        this.execute(task);
        return task;
    }
    
    @Override
    public <T> Future<T> submit(Callable<T> callable) {
        if (callable == null) {
            throw new NullPointerException();
        }
        RunnableFuture<T> task = this.newTaskFor(callable);
        this.execute(task);
        return task;
    }
    
    private <T> T doInvokeAny(Collection<? extends Callable<T>> collection, boolean b, long n) throws InterruptedException, ExecutionException, TimeoutException {
        if (collection == null) {
            throw new NullPointerException();
        }
        int size = collection.size();
        if (size == 0) {
            throw new IllegalArgumentException();
        }
        ArrayList list = new ArrayList(size);
        ExecutorCompletionService executorCompletionService = new ExecutorCompletionService(this);
        try {
            Object o = null;
            long n2 = b ? (System.nanoTime() + n) : 0L;
            Iterator<Object> iterator = collection.iterator();
            list.add(executorCompletionService.submit((Callable)iterator.next()));
            --size;
            int n3 = 1;
            while (true) {
                Future<Object> future = executorCompletionService.poll();
                if (future == null) {
                    if (size > 0) {
                        --size;
                        list.add(executorCompletionService.submit((Callable)iterator.next()));
                        ++n3;
                    }
                    else if (n3 == 0) {
                        if (o == null) {
                            o = new ExecutionException();
                        }
                        throw o;
                    }
                    else if (b) {
                        future = executorCompletionService.poll(n, TimeUnit.NANOSECONDS);
                        if (future == null) {
                            throw new TimeoutException();
                        }
                        n = n2 - System.nanoTime();
                    }
                    else {
                        future = executorCompletionService.take();
                    }
                }
                if (future != null) {
                    --n3;
                    try {
                        Object value = future.get();
                        for (int i = 0; i < list.size(); ++i) {
                            ((Future)list.get(i)).cancel(true);
                        }
                        return value;
                    }
                    catch (ExecutionException ex) {
                        o = ex;
                    }
                    catch (RuntimeException ex2) {
                        o = new ExecutionException(ex2);
                    }
                }
                else {
                    continue;
                }
            }
        }
        finally {
            for (int j = 0; j < list.size(); ++j) {
                ((Future)list.get(j)).cancel(true);
            }
        }
    }
    
    @Override
    public <T> T invokeAny(Collection<? extends Callable<T>> collection) throws InterruptedException, ExecutionException {
        try {
            return this.doInvokeAny(collection, false, 0L);
        }
        catch (TimeoutException ex) {
            assert false;
            return null;
        }
    }
    
    @Override
    public <T> T invokeAny(Collection<? extends Callable<T>> collection, long n, TimeUnit timeUnit) throws InterruptedException, ExecutionException, TimeoutException {
        return this.doInvokeAny(collection, true, timeUnit.toNanos(n));
    }
    
    @Override
    public <T> List<Future<T>> invokeAll(Collection<? extends Callable<T>> collection) throws InterruptedException {
        if (collection == null) {
            throw new NullPointerException();
        }
        ArrayList list = new ArrayList(collection.size());
        boolean n = false;
        try {
            Iterator<Object> iterator = collection.iterator();
            while (iterator.hasNext()) {
                RunnableFuture<Object> task = this.newTaskFor((Callable)iterator.next());
                list.add(task);
                this.execute(task);
            }
            for (int i = 0; i < list.size(); ++i) {
                Future<Object> future = (Future)list.get(i);
                if (!future.isDone()) {
                    try {
                        future.get();
                    }
                    catch (CancellationException ex) {}
                    catch (ExecutionException ex2) {}
                }
            }
            n = true;
            if (n == 0) {
                for (int j = 0; j < list.size(); ++j) {
                    ((Future)list.get(j)).cancel(true);
                }
                return list;
            }
            return list;
        }
        finally {
            if (n == 0) {
                for (int k = 0; k < list.size(); ++k) {
                    ((Future)list.get(k)).cancel(true);
                }
            }
        }
    }
    
    @Override
    public <T> List<Future<T>> invokeAll(Collection<? extends Callable<T>> collection, long n, TimeUnit timeUnit) throws InterruptedException {
        if (collection == null) {
            throw new NullPointerException();
        }
        long nanos = timeUnit.toNanos(n);
        ArrayList list = new ArrayList(collection.size());
        int n2 = 0;
        try {
            Iterator<Object> iterator = collection.iterator();
            while (iterator.hasNext()) {
                list.add(this.newTaskFor((Callable)iterator.next()));
            }
            long n3 = System.nanoTime() + nanos;
            int size = list.size();
            int i = 0;
            while (i < size) {
                this.execute((Runnable)list.get(i));
                nanos = n3 - System.nanoTime();
                if (nanos <= 0L) {
                    if (n2 == 0) {
                        for (int j = 0; j < list.size(); ++j) {
                            ((Future)list.get(j)).cancel(true);
                        }
                        return list;
                    }
                    return list;
                }
                else {
                    ++i;
                }
            }
            for (int k = 0; k < size; ++k) {
                Future<Object> future = (Future)list.get(k);
                if (!future.isDone()) {
                    if (nanos <= 0L) {
                        if (n2 == 0) {
                            for (int index = 0; index < list.size(); ++index) {
                                ((Future)list.get(index)).cancel(true);
                            }
                            return list;
                        }
                        return list;
                    }
                    else {
                        try {
                            future.get(nanos, TimeUnit.NANOSECONDS);
                        }
                        catch (CancellationException ex) {}
                        catch (ExecutionException ex2) {}
                        catch (TimeoutException ex3) {
                            if (n2 == 0) {
                                for (int l = 0; l < list.size(); ++l) {
                                    ((Future)list.get(l)).cancel(true);
                                }
                                return list;
                            }
                            return list;
                        }
                        nanos = n3 - System.nanoTime();
                    }
                }
            }
            n2 = 1;
            if (n2 == 0) {
                for (int index2 = 0; index2 < list.size(); ++index2) {
                    ((Future)list.get(index2)).cancel(true);
                }
                return list;
            }
            return list;
        }
        finally {
            if (n2 == 0) {
                for (int index3 = 0; index3 < list.size(); ++index3) {
                    ((Future)list.get(index3)).cancel(true);
                }
            }
        }
    }
}
